<?php
/**
* Yahoo Stock Quote module
* This module allows you to add the Stock Quote from Yahoo Finance in a module position.
* Author: kksou
* Copyright (C) 2006-2008. kksou.com. All Rights Reserved
* Website: http://www.kksou.com/php-gtk2
* v1.5 September 1, 2008
* v1.5.6 fixed line 297 print "<tr valign=top class=\"$row\">"; $row is redundant
* v1.57 new features
* - read yahoo data by CURL, file_get_contents() or fread()
* - display of symbol name from Yahoo e.g. ^N225 is NIKKEI 225, ^HSI is HANG SENG INDEX
* - added a lot more financial info: P/E, EPS, market cap, etc.
* - date format e.g. j/n/Y (the default), n/j
* - added global time offset
* - added global time zone name
* - provides link to different charts: 1d, 5d, 1m, ytd, 1y, 2y, 5y
* - now links to yahoo interactive chart instead of basic chart
* - allow users to set the order of the display fields
*/

require_once('yahoo_code.php');

class YahooStockQuote_Display {

	function YahooStockQuote_Display($lib_base, $style, $aparam, $joomla_ver) {
		$this->lib_base = $lib_base;
		$this->style = $style;
		$this->aparam = $aparam;

		$this->joomla_ver = $joomla_ver;
		if (isset($_GET['joomla_ver'])) $this->joomla_ver = $_GET['joomla_ver'];

		if (isset($_GET['aparam'])) {
			$a = explode('|', $_GET['aparam']);
			$this->aparam = new stdClass;
			foreach($a as $item) {
				list($k, $v) = explode('~', $item);
				$this->aparam->$k = $v;
			}
		}

		$this->yahoo_query = $this->get_yahoo_code();

		if (isset($_GET['lib_base']) && $_GET['lib_base']!='') $this->lib_base = $_GET['lib_base'];

		$this->data = array();

		$use_curl = 1;
		if (isset($aparam->use_curl)) $use_curl = $aparam->use_curl;
		if (isset($_GET['use_curl'])) $use_curl = $_GET['use_curl'];
		if (!preg_match('/\d+/', $use_curl)) $use_curl = 1;
		if ($use_curl=='') $use_curl = 1;
		if ($use_curl==3) {
			$app = new YahooStockQuote3();
		} elseif ($use_curl==2) {
			$app = new YahooStockQuote2();
		} else {
			$app = new YahooStockQuote();
		}
		$app->parent = $this;

		$s = '';
		if (isset($_GET["a"])) $s = $_GET["a"];

		$global_time_offset = $this->aparam->global_time_offset;
		$global_timezone_name = $this->aparam->global_timezone_name;
		if ($global_time_offset=='') $global_time_offset = 0;

		$this->time_offset = array();
		$this->time_offset_name = array();
		$s2 = '';
		foreach(explode(',', $s) as $item) {
			$item = trim(strtoupper($item));
			if (preg_match('/(.*)\(([+-\s]?\d+)([^)]*)\)/i', $item, $matches)) {
				$symbol = trim($matches[1]);
				$this->time_offset[$symbol] = trim($matches[2]);
				$this->time_offset_name[$symbol] = trim($matches[3]);
			} else {
				$symbol = $item;
				$this->time_offset[$symbol] = 0;
				$this->time_offset_name[$symbol] = '';
				if ($global_time_offset!=0) {
					$this->time_offset[$symbol] = $global_time_offset;
					$this->time_offset_name[$symbol] = $global_timezone_name;
				}
			}
			$s2 .= ','.$symbol;
		}
		if ($s2!='') $s2 = substr($s2, 1);

		$s = str_replace(',', ' ', $s2);
		$s = preg_replace('/\s+/', '+', $s);
		if ($s=='') $s = $this->aparam->default_symbols;
		if ($s=='') $s="GOOG,MSFT,YHOO";

		#$url = "http://finance.yahoo.com/d/quotes.csv?f=sl1d1t1c1ohgv&s=$s";
		$url = "http://finance.yahoo.com/d/quotes.csv?f=".$this->yahoo_query."&s=$s";
		$app->get_page($url);
		$this->data = $app->process();

		$this->setup_var();
		$this->setup_form();
	}

	function setup_var() {
		$this->str_a = '';
		if (isset($_GET["a"])) $this->str_a = $_GET['a'];
		if ($this->str_a=='') $this->str_a = $this->aparam->default_symbols;
		if ($this->str_a=='') $this->str_a = 'GOOG,MSFT,YHOO';

		if (isset($this->aparam->submit_button_label)) $this->submit_button_label = $this->aparam->submit_button_label;
		if (isset($_GET['submit_button_label'])) $this->submit_button_label = $_GET['submit_button_label'];
		if ($this->submit_button_label=='') $this->submit_button_label = 'Get Quotes';

		if (isset($this->aparam->input_size)) $this->input_size = $this->aparam->input_size;
		if ($this->input_size<4) $this->input_size = 4;

		$this->fields = array('symbol', 'date', 'time', 'trade', 'change', 'diff', 'open', 'high', 'low', 'volume', 'prev', 'intraday');
		$this->label_list = array('Symbol', 'Date', 'Time', 'Trade', 'Change', '% Chg', 'Open', 'High', 'Low', 'Volume', 'Prev Close', 'Intraday');

		$k = 0;
		$this->fields_label = array();
		foreach($this->fields as $var) {
			$this->fields_label[$var] = $this->label_list[$k];
			++$k;
		}

		$k = 0;
		$this->fields_id = array();
		foreach($this->fields as $var) {
			$this->fields_id[$var] = $k;
			++$k;
		}

		$this->display_list = array('symbol');
		foreach($this->fields as $var) {
			if (isset($this->aparam->$var) && $this->aparam->$var==1) $this->display_list[] = $var;
		}
		if (count($this->display_list)==1) $this->display_list = array('symbol', 'trade', 'change', 'diff', 'intraday');
	}

	function setup_form() {

		print "<link rel=\"stylesheet\" href=\"$this->lib_base/yahoostockquote.css\" type=\"text/css\" />";

		print '<style type="text/css">
<!--
div.yahoostock td.z1{
	text-align: right;
}
-->
</style>';

		print '<form action="" method="get" name="yahoostock_form" id="yahoostock_form">';
		print "<span class=\"yahoostock\">";

		$symbols = $this->str_a;
		$symbols = preg_replace('/(.*?)\s*\((\s+)([+-]?\d+[^)]*)\)/i', '$1(+$3)', $symbols);

		if ($this->aparam->hide_searchfield) {
			print "<input name=\"a\" type=\"hidden\" class=\"inputbox\" maxlength=\"255\" size=\"$this->input_size\" value=\"$symbols\">";
		} else {
			print "<input name=\"a\" class=\"inputbox\" maxlength=\"255\" size=\"$this->input_size\" value=\"$symbols\">";
			
			print "<span class=\"readon-wrap1\">";
			print "<span class=\"readon1-l\"></span>";
			print "<a class=\"readon1-main\">";
			print "<span class=\"readon1-m\">";
			print "<span class=\"readon1-r\">";
			print "<input type=\"submit\" name=\"submit\" value=\"$this->submit_button_label\" class=\"button\">&nbsp;";
			print "</span>";
			print "</span>";
			print "</a>";
			print "</span>";
			print "<p style=\"height:30px;\">";
			print "<p style=\"clear:both;\">";
		}
		

		if (count($this->data)>0) {
			$this->display();
		}

		$use_css = 1;
		print "<input type=\"hidden\" name=\"use_css\" value=\"$use_css\">";
		print "<input type=\"hidden\" name=\"lib_base\" value=\"$this->lib_base\">";

		print "<input type=\"hidden\" name=\"process\" value=\"1\">";

		$option = $this->getvar('option');
		$task = $this->getvar('task');
		$id = $this->getvar('id');
		$sectionid = $this->getvar('sectionid');
		$catid = $this->getvar('catid');
		$Itemid = $this->getvar('Itemid');

		print "<input type=\"hidden\" name=\"option\" value=\"$option\">";
		print "<input type=\"hidden\" name=\"task\" value=\"$task\">";
		print "<input type=\"hidden\" name=\"id\" value=\"$id\">";
		print "<input type=\"hidden\" name=\"sectionid\" value=\"$sectionid\">";
		print "<input type=\"hidden\" name=\"catid\" value=\"$catid\">";
		print "<input type=\"hidden\" name=\"Itemid\" value=\"$Itemid\">";

		$a = '';
		global $yahoo_code;
		foreach($yahoo_code as $k=>$v) {
			if (isset($this->aparam->$k)) {
				$z = $this->aparam->$k;
				if ($z!=0) $a .= "|$k~".$this->aparam->$k;
			}
		}

		$additional_fields = array('style', 'intraday', 'input_size', 'default_symbols', 'dateformat', 'global_time_offset', 'global_timezone_name', 'hide_searchfield');
		foreach($additional_fields as $item) {
			$a .= "|$item~".$this->aparam->$item;
		}

		$a = substr($a, 1);
		print "<input type=\"hidden\" name=\"aparam\" value=\"$a\">";

		print '</span>';
		print '</form>';
	}

	function getvar($var) {
		$val='';
		if (isset($_REQUEST[$var])) $val = $_REQUEST[$var];
		return $val;
	}

	function get_val($item, $k) {
		$class = '';
		$val = '';
		$align = '';
		if ($item=='symbol') {
			$class = 'symbol';
			$val = "<a href=\"http://finance.yahoo.com/q?s=$k\" target=\"_blank\"><span class=symbol>$k</span></a>";
		} elseif ($item=='change' || $item=='diff') {
			if ($this->data[$k][$item]>=0) {
				$gif = $this->lib_base.'/arrow33.gif';
				$style1 = 'data0';
			} else {
				$gif = $this->lib_base.'/arrow33a.gif';
				$style1 = 'data1';
			}
			$v = $this->data[$k][$item];
			if ($item=='change') $v = number_format($v, 2);
			if ($item=='diff' && preg_match('/^\+(.*)/', $v, $matches)) $v = $matches[1];
			if ($this->data[$k][$item]==0) {
				$class = "data";
				$val = $v;
			} else {
				$class = "$style1";
				$val = "<img class=\"arrow\" src=\"$gif\" alt=\"\" border=0 />$v";
			}
		} elseif ($item=='name') {
			$class = 'data_text';
			$val = $this->data[$k][$item];
		} elseif (preg_match('/chart_((\d)(d|m|y))/i', $item, $matches) || preg_match('/chart_(ytd)/i', $item, $matches)) {
			$chart_type = $matches[1];
			$class = 'data_text';
			#$val = "<a href=\"http://finance.yahoo.com/q/bc?s=$k&amp;t=$chart_type\" target=\"_blank\">view</a>";
			$val = "<a href=\"http://finance.yahoo.com/echarts?s=$k#symbol=$k;range=$chart_type\" target=\"_blank\">view</a>";
		} elseif ($item=='intraday') {
			$class = 'graph';
			if ($this->style==1) $align = 'right';
			#$val = "<a href=\"http://finance.yahoo.com/q/bc?s=$k&amp;t=1d\" target=\"_blank\"><img src=\"http://ichart.finance.yahoo.com/h?s=$k\" border=\"0\"></a>";
			$val = "<a href=\"http://finance.yahoo.com/echarts?s=$k#symbol=$k;range=1d\" target=\"_blank\"><img src=\"http://ichart.finance.yahoo.com/h?s=$k\" border=\"0\"></a>";
		} elseif ($item=='date' || $item=='time') {
			$class = "data";
			$dateformat = $this->aparam->dateformat;
			if (trim($dateformat)=='') $dateformat = 'n/j/Y';

			$t1 = explode('/', $this->data[$k]['date']);
			$ymd = $t1[2].'-'.$t1[0].'-'.$t1[1];

			if (isset($this->time_offset[$k]) && ($this->time_offset[$k]!=0)) {
				$t21 = date('Y-m-d H:i:s', strtotime("$ymd ".$this->data[$k]['time']));
				$offset = intval($this->time_offset[$k]);
				if ($offset>0) $offset = '+'.$offset;
				$t22_str = "$t21 $offset hours";
				$t22 = date('g:ia', strtotime($t22_str));
				$t22_date = date($dateformat, strtotime($t22_str));
				if ($item=='date') {
					$val = $t22_date;
				} else {
					$val = $t22.'&nbsp;'.$this->time_offset_name[$k];
				}
			} else {
				if ($item=='date') {
					#$val = $this->data[$k][$item];
					$val = date($dateformat, strtotime("$ymd 12:01:01"));
				} else {
					$val = $this->data[$k][$item].'&nbsp;ET';
				}
			}
		} else {
			$class = "data";
			$val = $this->data[$k][$item];
			if (in_array($item, array('volume'))) {
				$val = number_format($val, 0);
			} elseif (in_array($item, array('trade', 'change', 'diff', 'open', 'high', 'low', 'prev', 'volume'))) {
				$val = number_format($val, 2);
			}
		}
		return array($class, $val, $align);
	}

	function z1($n) {
		print "\n\n<!--
Changing the \"Powered by\" credit sentence without purchasing a licence is illegal!\n
Please visit http://www.kksou.com/php-gtk2/Joomla-Gadgets/Yahoo-Stock-Quote-AJAX-version.php#download for more information.
-->\n";

		print "<tr><td colspan=\"$n\" align=\"right\" class=\"z1\"><a href=\"http://www.kksou.com/php-gtk2/Joomla-Gadgets/Yahoo-Stock-Quote-AJAX-version.php\" style=\"color:#aaa;text-decoration: none;font-family:Tahoma, Arial, Helvetica, sans-serif;font-size:7pt;font-weight: normal;\">Powered by JoomlaGadgets</a></td></tr>";
	}

	function get_yahoo_code() {
		$this->aparam2 = array();
		foreach($this->aparam as $k=>$v) {
			if ($v!=0) $this->aparam2[$k] = $v;
		}
		$this->aparam2['symbol'] = -999999;

		asort($this->aparam2);

		global $yahoo_code;
		$this->aparam3 = array();
		$this->aparam3_yahoo = array();
		$yahoo_query = '';
		foreach($this->aparam2 as $k=>$v) {
			if (isset($yahoo_code[$k])) {
				$this->aparam3[] = $k;
				$v = $yahoo_code[$k];
				$yahoo_query .= $v[0];
				if ($v[0]!='') $this->aparam3_yahoo[] = $k;
			}
		}

		if (count($this->aparam3)==1) {
			print ">>>135!!!<br>";
			$yahoo_query = 'sl1c1p2';
			$this->aparam3 = array('symbol', 'trade', 'change', 'diff', 'intraday');
			$this->aparam3_yahoo = array('symbol', 'trade', 'change', 'diff');
		}

		return $yahoo_query;
	}

}

class YahooStockQuote_Style1 extends YahooStockQuote_Display {

	function display() {

		$symbols = array();
		foreach($this->data as $k=>$v) $symbols[] = $k;

		print "<table border=0>";

		global $yahoo_code;
		$i = 0;
		foreach($this->aparam3 as $item) {

			$v = $yahoo_code[$item];
			$label = $v[1];

			$row = 'd'.($i%2);
			if ($item=='symbol') {
				print "<tr valign=top class=\"$row\"><th class=symbol>&nbsp</th>";
			} else {
				print "<tr valign=top class=\"$row\"><td align=left class=label>$label&nbsp;</td>";
			}

			foreach($symbols as $k) {
				list($class, $val, $align) = $this->get_val($item, $k);
				if ($item=='symbol') {
					print "<th class=\"$class\">$val</th>";
				} else {
					if ($align!='') {
						print "<td align=\"$align\" class=\"$class\">$val</td>";
					} else {
						print "<td class=\"$class\">$val</td>";
					}
				}
			}

			print "</tr>";
			++$i;

		}

		$this->z1(count($symbols)+1);

		print "</table>";

	}
}

class YahooStockQuote_Style2 extends YahooStockQuote_Display {

	function display() {

		$symbols = array();
		foreach($this->data as $k=>$v) $symbols[] = $k;

		global $yahoo_code;
		print "<table border=0>";
		print "<tr valign=top>";
		foreach($this->aparam3 as $item) {
			$v = $yahoo_code[$item];
			$label = $v[1];
			print "<th align=\"center\" class=symbol>$label</th>";
		}

		$i = 1;
		foreach($symbols as $k) {
			$row = 'd'.($i%2);
			print "<tr valign=top class=\"$row\">";
			foreach($this->aparam3 as $item) {
				$v = $yahoo_code[$item];
				list($class, $val, $align) = $this->get_val($item, $k);
				print "<td class=\"$class\">$val</td>";
			}

			++$i;
			print "</tr>";
		}

		$this->z1(count($this->aparam3));
		print "</table>";
	}
}

class YahooStockQuote_base {

	var $ch;
	var $html;
	var $binary = 0;
	var $url;

	function __construct() {
		$this->html = "";
	}

	function process() {

		$data2 = array();
		for ($i=0; $i<count($this->data); ++$i) {
			$v = $this->data[$i];
			$symbol = $v[0];
			for($j=1; $j<count($v); ++$j) {
				$data2[$symbol][$this->parent->aparam3_yahoo[$j]] = $v[$j];
			}
		}
		return $data2;
	}

	function process_csv() {
		$data = $this->html;

		#$regexp = '^("[^"\n]*"|[^,\n]*),("[^"\n]*"|[^,\n]*),("[^"\n]*"|[^,\n]*),("[^"\n]*"|[^,\n]*),("[^"\n]*"|[^,\n]*),("[^"\n]*"|[^,\n]*),("[^"\n]*"|[^,\n]*),("[^"\n]*"|[^,\n]*),("[^"\n]*"|[^,\n]*)$';

		$regexp = '';
		for ($i=0; $i<count($this->parent->aparam3_yahoo); ++$i) {
			$regexp .= ',("[^"\n]*"|[^,\n]*)';
		}
		$regexp = '^'.substr($regexp, 1).'$';

		$regexp2 = str_replace('\n', '\r\n', $regexp);

		$data2 = array();
		$ok = 0;
		if (preg_match_all("/$regexp/m", $data, $matches_array, PREG_SET_ORDER)) {
			$ok = 1;
		} elseif (preg_match_all("/$regexp2/m", $data, $matches_array, PREG_SET_ORDER)) {
			$ok = 1;
		}

		$data = array();
		for ($i=0; $i<count($matches_array); ++$i) {
			$v = $matches_array[$i];
			$item = array();
			for($j=1; $j<count($v); ++$j) {
				$str = $v[$j];
				if (preg_match('/^"(.*)"$/m', $str, $matches2)) $str = $matches2[1];
				$item[] = $str;
			}
			$data[] = $item;
		}
		return $data;
	}
}

class YahooStockQuote extends YahooStockQuote_base {
	function get_page($url) {
		if ($url!='') {
			$ch = curl_init ();
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt($ch, CURLOPT_URL, $url);
			curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
			curl_setopt($ch, CURLOPT_BINARYTRANSFER, $this->binary);
			$this->html = curl_exec($ch);
			curl_close($ch);
			$this->data = $this->process_csv();
		}
	}
}

class YahooStockQuote2 extends YahooStockQuote_base {
	function get_page($url) {
		if ($url!='') {
			$this->html = file_get_contents($url);
			$this->data = $this->process_csv();
		}
	}
}

class YahooStockQuote3 extends YahooStockQuote_base {
	function get_page($url) {
		if ($url!='') {
			$fp = fopen($url, "r");
			$this->data = array();
			while (($data = fgetcsv($fp, 32768, ",")) !== FALSE) {
				$this->data[] = $data;
			}
			fclose($fp);
		}
	}
}


if (isset($aparam->style)) $style = $aparam->style;
if (isset($_GET['style'])) $style = $_GET['style'];
if (!preg_match('/\d+/', $style)) $style = 1;
if ($style=='') $style = 1;
$class = "YahooStockQuote_Style".$style;
if (!isset($aparam)) $aparam = new stdClass;
if (isset($_GET['lib_base']) && $_GET['lib_base']!='') $lib_base = $_GET['lib_base'];
if (!isset($lib_base)) $lib_base = '';
if (!isset($joomla_ver)) $joomla_ver = 0;
$app = new $class($lib_base, $style, $aparam, $joomla_ver);
?>
